/*
 ***********************************************************************
 *                 C++ Wildcard Pattern Matching Library               *
 *                                                                     *
 * Author: Arash Partow (2001)                                         *
 * URL: https://www.partow.net/programming/wildcardmatching/index.html *
 *                                                                     *
 * Copyright notice:                                                   *
 * Free use of the C++ Wildcard Pattern Matching Library is permitted  *
 * under the guidelines and in accordance with the most current        *
 * version of the MIT License.                                         *
 * https://www.opensource.org/licenses/MIT                             *
 *                                                                     *
 ***********************************************************************
*/


#include <cstdio>
#include <chrono>
#include <string>

#include "globmatch.hpp"


static const std::pair<std::string,std::string> tests[] =
               {
                  { ""            , ""          },
                  { "a"           , "a*"        },
                  { "a"           , "a."        },
                  { "aaa"         , "*aa."      },
                  { "aaaa"        , "*aa."      },
                  { "aaaaa"       , "*aa."      },
                  { "aaaaaa"      , "*aa."      },
                  { "aaaaabbbbb"  , "*aa.*bb."  },
                  { "ab"          , "a*"        },
                  { "ab"          , "a."        },
                  { "abcd"        , "***d"      },
                  { "abcd"        , "**cd"      },
                  { "abcd"        , "*.cd"      },
                  { "abcd"        , "*b*d"      },
                  { "abcd"        , "*b.*"      },
                  { "abcd"        , "*b.."      },
                  { "abcd"        , "*bc*"      },
                  { "abcd"        , "*bc."      },
                  { "abcd"        , "*bcd"      },
                  { "abcd"        , ".*.d"      },
                  { "abcd"        , ".*cd"      },
                  { "abcd"        , "...."      },
                  { "abcd"        , "....."     },
                  { "abcd"        , "...d"      },
                  { "abcd"        , "..c*"      },
                  { "abcd"        , "..cd"      },
                  { "abcd"        , ".b.d"      },
                  { "abcd"        , ".bc*"      },
                  { "abcd"        , ".bc."      },
                  { "abcd"        , ".bcd"      },
                  { "abcd"        , "a*"        },
                  { "abcd"        , "a***"      },
                  { "abcd"        , "a*.d"      },
                  { "abcd"        , "a*c*"      },
                  { "abcd"        , "a*cd"      },
                  { "abcd"        , "a.*d"      },
                  { "abcd"        , "a..."      },
                  { "abcd"        , "a...."     },
                  { "abcd"        , "a..d"      },
                  { "abcd"        , "a.c."      },
                  { "abcd"        , "a.cd"      },
                  { "abcd"        , "ab*"       },
                  { "abcd"        , "ab**"      },
                  { "abcd"        , "ab*."      },
                  { "abcd"        , "ab*d"      },
                  { "abcd"        , "ab.*"      },
                  { "abcd"        , "ab.."      },
                  { "abcd"        , "ab..."     },
                  { "abcd"        , "ab.d"      },
                  { "abcd"        , "abc*"      },
                  { "abcd"        , "abc."      },
                  { "abcd"        , "abc.."     },
                  { "abcd"        , "abcd"      },
                  { "abcd"        , "abcd*"     },
                  { "abcd"        , "abcd."     },
                  { "xabclabcd"   , "*abcd*"    },
                  { "xablabcd"    , "*abcd*"    },
                  { "xalabcd"     , "*abcd*"    },
                  { "aabbbbabb"   , ".aa.*abb." },
                  { "abc"         , "xyzw"      },
                  { "abcd"        , "a."        },
                  { "abcd"        , "ab."       },
                  { "abcd"        , "xyz"       },
                  { "abcd"        , "xyzw"      },
                  { "abcxbcdx"    , "*abcd*"    },
                  { "xabcxbcdx"   , "*abcd*"    },
                  { "axbxcxd"                                                    , "a*b*c*d" },
                  { "axxbxxcxxd"                                                 , "a*b*c*d" },
                  { "axxxbxxxcxxxd"                                              , "a*b*c*d" },
                  { "axxxxbxxxxcxxxxd"                                           , "a*b*c*d" },
                  { "axxxxxbxxxxxcxxxxxd"                                        , "a*b*c*d" },
                  { "axxxxxxbxxxxxxcxxxxxxd"                                     , "a*b*c*d" },
                  { "axxxxxxxbxxxxxxxcxxxxxxxd"                                  , "a*b*c*d" },
                  { "axxxxxxxxbxxxxxxxxcxxxxxxxxd"                               , "a*b*c*d" },
                  { "axxxxxxxxxbxxxxxxxxxcxxxxxxxxxd"                            , "a*b*c*d" },
                  { "axxxxxxxxxxbxxxxxxxxxxcxxxxxxxxxxd"                         , "a*b*c*d" },
                  { "axxxxxxxxxxxbxxxxxxxxxxxcxxxxxxxxxxxd"                      , "a*b*c*d" },
                  { "axxxxxxxxxxxxbxxxxxxxxxxxxcxxxxxxxxxxxxd"                   , "a*b*c*d" },
                  { "axxxxxxxxxxxxxbxxxxxxxxxxxxxcxxxxxxxxxxxxxd"                , "a*b*c*d" },
                  { "axxxxxxxxxxxxxxbxxxxxxxxxxxxxxcxxxxxxxxxxxxxxd"             , "a*b*c*d" },
                  { "axxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxd"          , "a*b*c*d" },
                  { "axxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxd"       , "a*b*c*d" },
                  { "axxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxd"    , "a*b*c*d" },
                  { "axxxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxxd" , "a*b*c*d" },
                  { "axbxcxd"                                                    , "a.*b.*c.*d" },
                  { "axxbxxcxxd"                                                 , "a.*b.*c.*d" },
                  { "axxxbxxxcxxxd"                                              , "a.*b.*c.*d" },
                  { "axxxxbxxxxcxxxxd"                                           , "a.*b.*c.*d" },
                  { "axxxxxbxxxxxcxxxxxd"                                        , "a.*b.*c.*d" },
                  { "axxxxxxbxxxxxxcxxxxxxd"                                     , "a.*b.*c.*d" },
                  { "axxxxxxxbxxxxxxxcxxxxxxxd"                                  , "a.*b.*c.*d" },
                  { "axxxxxxxxbxxxxxxxxcxxxxxxxxd"                               , "a.*b.*c.*d" },
                  { "axxxxxxxxxbxxxxxxxxxcxxxxxxxxxd"                            , "a.*b.*c.*d" },
                  { "axxxxxxxxxxbxxxxxxxxxxcxxxxxxxxxxd"                         , "a.*b.*c.*d" },
                  { "axxxxxxxxxxxbxxxxxxxxxxxcxxxxxxxxxxxd"                      , "a.*b.*c.*d" },
                  { "axxxxxxxxxxxxbxxxxxxxxxxxxcxxxxxxxxxxxxd"                   , "a.*b.*c.*d" },
                  { "axxxxxxxxxxxxxbxxxxxxxxxxxxxcxxxxxxxxxxxxxd"                , "a.*b.*c.*d" },
                  { "axxxxxxxxxxxxxxbxxxxxxxxxxxxxxcxxxxxxxxxxxxxxd"             , "a.*b.*c.*d" },
                  { "axxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxd"          , "a.*b.*c.*d" },
                  { "axxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxd"       , "a.*b.*c.*d" },
                  { "axxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxd"    , "a.*b.*c.*d" },
                  { "axxxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxxd" , "a.*b.*c.*d" },
                  { "axbxcxd"                                                    , "a*.b*.c*.d" },
                  { "axxbxxcxxd"                                                 , "a*.b*.c*.d" },
                  { "axxxbxxxcxxxd"                                              , "a*.b*.c*.d" },
                  { "axxxxbxxxxcxxxxd"                                           , "a*.b*.c*.d" },
                  { "axxxxxbxxxxxcxxxxxd"                                        , "a*.b*.c*.d" },
                  { "axxxxxxbxxxxxxcxxxxxxd"                                     , "a*.b*.c*.d" },
                  { "axxxxxxxbxxxxxxxcxxxxxxxd"                                  , "a*.b*.c*.d" },
                  { "axxxxxxxxbxxxxxxxxcxxxxxxxxd"                               , "a*.b*.c*.d" },
                  { "axxxxxxxxxbxxxxxxxxxcxxxxxxxxxd"                            , "a*.b*.c*.d" },
                  { "axxxxxxxxxxbxxxxxxxxxxcxxxxxxxxxxd"                         , "a*.b*.c*.d" },
                  { "axxxxxxxxxxxbxxxxxxxxxxxcxxxxxxxxxxxd"                      , "a*.b*.c*.d" },
                  { "axxxxxxxxxxxxbxxxxxxxxxxxxcxxxxxxxxxxxxd"                   , "a*.b*.c*.d" },
                  { "axxxxxxxxxxxxxbxxxxxxxxxxxxxcxxxxxxxxxxxxxd"                , "a*.b*.c*.d" },
                  { "axxxxxxxxxxxxxxbxxxxxxxxxxxxxxcxxxxxxxxxxxxxxd"             , "a*.b*.c*.d" },
                  { "axxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxd"          , "a*.b*.c*.d" },
                  { "axxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxd"       , "a*.b*.c*.d" },
                  { "axxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxd"    , "a*.b*.c*.d" },
                  { "axxxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxxd" , "a*.b*.c*.d" },
                  { "axbxcxd"                                                    , "a.**b.**c.**d" },
                  { "axxbxxcxxd"                                                 , "a.**b.**c.**d" },
                  { "axxxbxxxcxxxd"                                              , "a.**b.**c.**d" },
                  { "axxxxbxxxxcxxxxd"                                           , "a.**b.**c.**d" },
                  { "axxxxxbxxxxxcxxxxxd"                                        , "a.**b.**c.**d" },
                  { "axxxxxxbxxxxxxcxxxxxxd"                                     , "a.**b.**c.**d" },
                  { "axxxxxxxbxxxxxxxcxxxxxxxd"                                  , "a.**b.**c.**d" },
                  { "axxxxxxxxbxxxxxxxxcxxxxxxxxd"                               , "a.**b.**c.**d" },
                  { "axxxxxxxxxbxxxxxxxxxcxxxxxxxxxd"                            , "a.**b.**c.**d" },
                  { "axxxxxxxxxxbxxxxxxxxxxcxxxxxxxxxxd"                         , "a.**b.**c.**d" },
                  { "axxxxxxxxxxxbxxxxxxxxxxxcxxxxxxxxxxxd"                      , "a.**b.**c.**d" },
                  { "axxxxxxxxxxxxbxxxxxxxxxxxxcxxxxxxxxxxxxd"                   , "a.**b.**c.**d" },
                  { "axxxxxxxxxxxxxbxxxxxxxxxxxxxcxxxxxxxxxxxxxd"                , "a.**b.**c.**d" },
                  { "axxxxxxxxxxxxxxbxxxxxxxxxxxxxxcxxxxxxxxxxxxxxd"             , "a.**b.**c.**d" },
                  { "axxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxd"          , "a.**b.**c.**d" },
                  { "axxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxd"       , "a.**b.**c.**d" },
                  { "axxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxd"    , "a.**b.**c.**d" },
                  { "axxxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxxd" , "a.**b.**c.**d" },
                  { "axbxcxd"                                                    , "a**.b**.c**.d" },
                  { "axxbxxcxxd"                                                 , "a**.b**.c**.d" },
                  { "axxxbxxxcxxxd"                                              , "a**.b**.c**.d" },
                  { "axxxxbxxxxcxxxxd"                                           , "a**.b**.c**.d" },
                  { "axxxxxbxxxxxcxxxxxd"                                        , "a**.b**.c**.d" },
                  { "axxxxxxbxxxxxxcxxxxxxd"                                     , "a**.b**.c**.d" },
                  { "axxxxxxxbxxxxxxxcxxxxxxxd"                                  , "a**.b**.c**.d" },
                  { "axxxxxxxxbxxxxxxxxcxxxxxxxxd"                               , "a**.b**.c**.d" },
                  { "axxxxxxxxxbxxxxxxxxxcxxxxxxxxxd"                            , "a**.b**.c**.d" },
                  { "axxxxxxxxxxbxxxxxxxxxxcxxxxxxxxxxd"                         , "a**.b**.c**.d" },
                  { "axxxxxxxxxxxbxxxxxxxxxxxcxxxxxxxxxxxd"                      , "a**.b**.c**.d" },
                  { "axxxxxxxxxxxxbxxxxxxxxxxxxcxxxxxxxxxxxxd"                   , "a**.b**.c**.d" },
                  { "axxxxxxxxxxxxxbxxxxxxxxxxxxxcxxxxxxxxxxxxxd"                , "a**.b**.c**.d" },
                  { "axxxxxxxxxxxxxxbxxxxxxxxxxxxxxcxxxxxxxxxxxxxxd"             , "a**.b**.c**.d" },
                  { "axxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxd"          , "a**.b**.c**.d" },
                  { "axxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxd"       , "a**.b**.c**.d" },
                  { "axxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxd"    , "a**.b**.c**.d" },
                  { "axxxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxxd" , "a**.b**.c**.d" },
                  { "axbxcxd"                                                    , "a.*b.*c.*d"                                                    },
                  { "axxbxxcxxd"                                                 , "a..*b..*c..*d"                                                 },
                  { "axxxbxxxcxxxd"                                              , "a...*b...*c...*d"                                              },
                  { "axxxxbxxxxcxxxxd"                                           , "a....*b....*c....*d"                                           },
                  { "axxxxxbxxxxxcxxxxxd"                                        , "a.....*b.....*c.....*d"                                        },
                  { "axxxxxxbxxxxxxcxxxxxxd"                                     , "a......*b......*c......*d"                                     },
                  { "axxxxxxxbxxxxxxxcxxxxxxxd"                                  , "a.......*b.......*c.......*d"                                  },
                  { "axxxxxxxxbxxxxxxxxcxxxxxxxxd"                               , "a........*b........*c........*d"                               },
                  { "axxxxxxxxxbxxxxxxxxxcxxxxxxxxxd"                            , "a.........*b.........*c.........*d"                            },
                  { "axxxxxxxxxxbxxxxxxxxxxcxxxxxxxxxxd"                         , "a..........*b..........*c..........*d"                         },
                  { "axxxxxxxxxxxbxxxxxxxxxxxcxxxxxxxxxxxd"                      , "a...........*b...........*c...........*d"                      },
                  { "axxxxxxxxxxxxbxxxxxxxxxxxxcxxxxxxxxxxxxd"                   , "a............*b............*c............*d"                   },
                  { "axxxxxxxxxxxxxbxxxxxxxxxxxxxcxxxxxxxxxxxxxd"                , "a.............*b.............*c.............*d"                },
                  { "axxxxxxxxxxxxxxbxxxxxxxxxxxxxxcxxxxxxxxxxxxxxd"             , "a..............*b..............*c..............*d"             },
                  { "axxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxd"          , "a...............*b...............*c...............*d"          },
                  { "axxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxd"       , "a................*b................*c................*d"       },
                  { "axxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxd"    , "a.................*b.................*c.................*d"    },
                  { "axxxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxxd" , "a..................*b..................*c..................*d" },
                  { "axbxcxd"                                                    , "a*.b*.c*.d"                                                    },
                  { "axxbxxcxxd"                                                 , "a*..b*..c*..d"                                                 },
                  { "axxxbxxxcxxxd"                                              , "a*...b*...c*...d"                                              },
                  { "axxxxbxxxxcxxxxd"                                           , "a*....b*....c*....d"                                           },
                  { "axxxxxbxxxxxcxxxxxd"                                        , "a*.....b*.....c*.....d"                                        },
                  { "axxxxxxbxxxxxxcxxxxxxd"                                     , "a*......b*......c*......d"                                     },
                  { "axxxxxxxbxxxxxxxcxxxxxxxd"                                  , "a*.......b*.......c*.......d"                                  },
                  { "axxxxxxxxbxxxxxxxxcxxxxxxxxd"                               , "a*........b*........c*........d"                               },
                  { "axxxxxxxxxbxxxxxxxxxcxxxxxxxxxd"                            , "a*.........b*.........c*.........d"                            },
                  { "axxxxxxxxxxbxxxxxxxxxxcxxxxxxxxxxd"                         , "a*..........b*..........c*..........d"                         },
                  { "axxxxxxxxxxxbxxxxxxxxxxxcxxxxxxxxxxxd"                      , "a*...........b*...........c*...........d"                      },
                  { "axxxxxxxxxxxxbxxxxxxxxxxxxcxxxxxxxxxxxxd"                   , "a*............b*............c*............d"                   },
                  { "axxxxxxxxxxxxxbxxxxxxxxxxxxxcxxxxxxxxxxxxxd"                , "a*.............b*.............c*.............d"                },
                  { "axxxxxxxxxxxxxxbxxxxxxxxxxxxxxcxxxxxxxxxxxxxxd"             , "a*..............b*..............c*..............d"             },
                  { "axxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxd"          , "a*...............b*...............c*...............d"          },
                  { "axxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxd"       , "a*................b*................c*................d"       },
                  { "axxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxd"    , "a*.................b*.................c*.................d"    },
                  { "axxxxxxxxxxxxxxxxxxbxxxxxxxxxxxxxxxxxxcxxxxxxxxxxxxxxxxxxd" , "a*..................b*..................c*..................d" }
               };

int main(int argc, char* argv[])
{
   if (argc == 4)
   {
      const std::size_t rounds  = atoi(argv[1]);
      const std::string pattern = argv[2];
      const std::string data    = argv[3];
      std::size_t match_count   = 0;

      const auto start = std::chrono::steady_clock::now();

      for (std::size_t i = 0; i < rounds; ++i)
      {
         if (glob::match(data,pattern))
            match_count++;
      }

      const auto end = std::chrono::steady_clock::now();

      const auto time_ns = std::chrono::duration <double, std::nano> (end - start).count();

      printf("pattern: %s data: %s matches: %lu time: %10.3fns rate: %10.3f mtchs/sec\n",
              pattern.c_str(),
              data.c_str(),
              match_count,
              time_ns,
              rounds / (time_ns / 1e9));

      return 0;
   }

   const std::size_t rounds = 1000000;
   const std::size_t N = sizeof(tests) / sizeof(std::pair<std::string,std::string>);

   std::size_t match_count = 0;

   const auto start = std::chrono::steady_clock::now();

   for (std::size_t r = 0; r < rounds; ++r)
   {
      for (std::size_t i = 0; i < N; ++i)
      {
         if (glob::match(tests[i].first,tests[i].second))
            match_count++;
      }
   }

   const auto end = std::chrono::steady_clock::now();

   const auto time_ns  = std::chrono::duration <double, std::nano> (end - start).count();

   printf("match count: %zu, total time: %10.5fns\trate: %15.4fmatches/sec\n",
           match_count,
           time_ns,
           (rounds * N) / (time_ns / 1e9));

   return 0;
}
